/* ccgo: go/go.cc
 * 
 * Copyright 2004 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <go.hh>
#include <debug.hh>
#include <iostream>
#include <sstream>
using namespace ccgo;
using namespace std;

// members of class Error
Error::Error(string message)
{
	s = message;
}

const string & Error::get_message()
{
	return s;
}

// members of class Board
Board::Board() :
	z(0),
	b(0)
{
}

Board::Board(int size) :
	z(0),
	b(0)	
{
	if (size < 0) throw Error("negative board size");
	z = size;
	if (z) b = new State[z * z];
	for (int i = z * z - 1; i >= 0; i --) b[i] = STATE_EMPTY;
}

Board::Board(const Board & board) : // copy constructor
	z(0),
	b(0)
{
	z = board.z;
	if (z) {
		b = new State[z * z];
		for (int i = z * z - 1; i >= 0; i --) b[i] = board.b[i];
	}
}

Board::~Board()
{
	if (b) delete b;
}

void Board::copy(const Board & board)
{
	if (b) delete b;
	z = board.z;
	if (z) {
		b = new State[z * z];
		for (int i = z * z - 1; i >= 0;i --) b[i] = board.b[i];
	}
}

void Board::clear()
{
	for (int i = z * z - 1; i >= 0; i --) b[i] = STATE_EMPTY;
}

int Board::get_size() const
{
	return z;
}

Board::State Board::get_state(int loc) const
{
	if (loc < 0 || loc >= z * z) throw Error("out or range loc");
	return b[loc];
}

void Board::set_state(int loc, State state)
{
	if (loc < 0 || loc >= z * z) throw Error("out or range loc");
	b[loc] = state;
}

vector<int> Board::neighbors(int loc) const
{
	if (loc < 0 || loc >= z * z) throw Error("out or range loc");
	vector<int> n;
	if (loc % z > 0) n.push_back(loc - 1);
	if (loc % z < z - 1) n.push_back(loc + 1);
	if (loc / z > 0) n.push_back(loc - z);
	if (loc / z < z - 1) n.push_back(loc + z);
	return n;
}

int Board::loc_range() const
{
	return z * z;
}

int Board::flip_axis(int x) const
{
	return z - x - 1;
}

int Board::xy_to_loc(int x, int y) const 
{
	if (x >= 0 && x < z && y >= 0 && y < z) return x + y * z;
	return NO_LOC;
}

void Board::loc_to_xy(int loc, int & x, int & y) const
{
	x = loc % z;
	y = loc / z;
}

void Board::get_kill(int loc, vector<int> & kill, State put_state) const
{
	if (loc < 0 || loc >= z * z) throw Error("out or range loc");
	int zz = z * z;
	bool m[zz]; // marking field
	State s = State(3 - put_state); // check other stones around
	vector<int> n = neighbors(loc); // checking the neighbors of l
	kill.clear();
	int bn = 0; // number of killed stones
	while (n.size()) {
		for (int i = 0; i < zz; i ++) m[i] = false; // clear marking field
		m[loc] = true; // fill the point!
		int q = qi_group(n.back(), kill, s, m);
		n.pop_back();
		for (vector<int>::iterator i = n.begin(); i != n.end(); i ++) if (m[* i]) {
			// remove checked neighbor from the list
			n.erase(i);
			i --;
		}
		if (q > 0) kill.resize(bn); // group is alive, remove from k
		else bn = kill.size();
	}
	if (bn > 0) return; // kill something

	// killed no one,  check for suicide
	for (int i = 0; i < zz; i ++) m[i] = false; // clear marking field
	m[loc] = true;
	kill.push_back(loc);
	int q = 0;
	n = neighbors(loc);
	s = put_state; // check same stones around
	while (n.size()) {
		q += qi_group(n.back(), kill, s, m);
		n.pop_back();
	}
	if (q) kill.clear(); // not suicide, clear k
}

int Board::qi_group(int loc, vector<int> & group, State state, bool * mark_field) const
{
	if (loc < 0 || loc >= z * z) throw Error("out or range loc");
	if (mark_field[loc]) return 0;
	mark_field[loc] = true; // mark the loc
	if (get_state(loc) == STATE_EMPTY) return 1;
	if (get_state(loc) != state) return 0;
	group.push_back(loc); // add the loc
	int q = 0;
	vector<int> n = neighbors(loc);
	while (n.size()) {
		q += qi_group(n.back(), group, state, mark_field); // recursion
		n.pop_back();
	}
	return q;
}

void Board::score_terri(Terri * terri) const
// terri should be setup by caller, with killed stones marked as territory
{
	int r = loc_range();
	bool * marks = new bool[r];
	for (int i = 0; i < r; i ++) {
		// clear TERRI except for the stones
		if (terri[i] && ! b[i]) terri[i] = TERRI_NEUTRAL;
		// clear marking board
		marks[i] = false;
	}
	for (int i = 0; i < r; i ++) if (! marks[i] && (! b[i] || terri[i])) { // empty || killed nonmarked site
		std::vector<int> l; // l should be empty
		bool w_seen = false; // white encountered
		bool b_seen = false; // black encountered
		find_terri(i, l, w_seen, b_seen, terri, marks); // explore the empty space
		Terri t = TERRI_NEUTRAL;
		if (w_seen && ! b_seen) t = TERRI_WHITE; // see white only -> white territory
		else if (! w_seen && b_seen) t = TERRI_BLACK; // -> black_territory
		for (std::vector<int>::const_iterator i = l.begin(); i != l.end(); i ++) if (! b[* i]) terri[* i] = t; // set terri except for stones
	}
	// the above is an over estimate, some empty points linking to hanging groups need to be filled (fake eyes)
        bool rr; // any changes? (iteratively eliminate fake eyes...
        do {
                rr = false;
                // for blacks
		for (int i = 0; i < r; i ++) marks[i] = false; // clear the marking board
                for (int i = 0; i < r; i ++) if (! marks[i] && b[i] == STATE_BLACK && ! terri[i]) { // a live black stone
			std::vector<int> qi; // territory qi points encounterred
			hang_group(i, STATE_WHITE, qi, terri, marks);
			if (qi.size() == 1 && ! b[qi[0]]) { // one qi only... need to be connected
				// unless it's removed stone point, then we will leave it along
				terri[qi[0]] = TERRI_NEUTRAL;
				rr = true;
			}
			while (qi.size() > 0) { // unmark the territory qi points, as they can be shared by groups
				marks[qi.back()] = false;
				qi.pop_back();
			}
		}
		// do the same for white
		for (int i = 0; i < r; i ++) marks[i] = false; // clear the marking board
		for (int i = 0; i < r; i ++) if (! marks[i] && b[i] == STATE_WHITE && ! terri[i]) {
			std::vector<int> qi;
			hang_group(i, STATE_BLACK, qi, terri, marks);
			if (qi.size() == 1 && ! b[qi[0]]) {
				terri[qi[0]] = TERRI_NEUTRAL;
				rr = true;
			}
			while (qi.size() > 0) {
				marks[qi.back()] = false;
				qi.pop_back();
			}
		}
	} while (rr);
	delete marks;
}

void Board::find_terri(int loc, std::vector<int> & group, bool & w_seen, bool & b_seen, Terri * terri, bool * marks) const
{
	if (b[loc] && ! terri[loc]) {
		if (b[loc] == STATE_WHITE) w_seen = true;
		else b_seen = true;
		return;
	}
	marks[loc] = true;
	group.push_back(loc);
	std::vector<int> n = neighbors(loc);
	for (std::vector<int>::iterator i = n.begin(); i != n.end(); i ++) {
		if (marks[* i]) continue;
		find_terri(* i, group, w_seen, b_seen, terri, marks);
	}
}

void Board::hang_group(int loc, State state, std::vector<int> & qi, Terri * terri, bool * marks) const
{
	if (terri[loc]) {
		qi.push_back(loc);
		marks[loc] = true;
		return;
	}
	if (b[loc] == state) return;
	marks[loc] = true;
	std::vector<int> n = neighbors(loc);
	for (std::vector<int>::iterator i = n.begin(); i != n.end(); i ++) {
		if (marks[* i]) continue;
		hang_group(* i, state, qi, terri, marks);
	}
}

void Board::get_group(int loc, std::vector<int> & group) const
{
	if (! b[loc]) return;
	int r = loc_range();
	bool * marks = new bool[r];
	for (int i = 0; i < r; i ++) marks[i] = false;
	qi_group(loc, group, b[loc], marks);
	delete marks;
}

void Board::dead_group(int loc, std::vector<int> & group) const
{
	assert(b[loc]);
	int r = loc_range();
	bool * marks = new bool[r];
	for (int i = 0; i < r; i ++) marks[i] = false;
	terri_group(loc, b[loc], group, marks);
	delete marks;
}

void Board::terri_group(int loc, State state, std::vector<int> & group, bool * marks) const
{
	marks[loc] = true;
	if (b[loc] == 3 - state) return;
	if (b[loc] == state) group.push_back(loc);
	std::vector<int> n = neighbors(loc);
	for (std::vector<int>::iterator i = n.begin(); i != n.end(); i ++) if (! marks[* i]) terri_group(* i, state, group, marks);
}

int Board::compact_size() const
{
	return (z * z + 4) / 5;
}

void Board::compact_get(void * data) const
{
	unsigned char * d = (unsigned char *) data;
	d --;
	int zz = z * z;
	for (int i = 0; i < zz; i ++) {
		if (i % 5 == 0) {
			d ++;
			* d = 0;
		}
		* d *= 3;
		* d += b[i];
	}
}

bool Board::loc_is_star(int loc) const
{
	if (loc < 0 || loc >= z * z) throw Error("out or range loc");
	int x = loc % z;
	int y = loc / z;
	if (z > 9) return (x == 3 || x == z - 4 || 2 * x == z - 1) && (y == 3 || y == z - 4 || 2 * y == z - 1);
	if (z == 9) return (x == 2 || x == z - 3 || 2 * x == z - 1) && (y == 2 || y == z - 3 || 2 * y == z - 1);
	// only 4 stars for board size 5~9
	if (z > 4) return (x == 2 || x == z - 3) && (y == 2 || y == z - 3);
	// no star for board size 4 and below
	return false;
}

void Board::set_handi(int handi)
{
	if (handi && handi <= 1) throw Error("too few handicap stones");
	if (handi > 9) throw Error("too many handicap stones");
	static const int hx[] = {- 1, 1, - 1, 1, - 1, 1, 0, 0};
	static const int hy[] = {- 1, 1, 1, - 1, 0, 0, - 1, 1};
	int n = handi;
	if (z < 5) throw Error("board too small for handicap");
	if (n > 4 && (z % 2 == 0 || z < 9)) throw Error("too many handicap stones for the board");
	clear(); // clear board
	if (n > 4) {
		if (n % 2 == 1) { // odd number handicap,put one at center
			-- n;
			set_state(z / 2 + (z / 2) * z, Board::STATE_BLACK);
		}
	}
	// place the rest handi stones in order
	int d = z > 9 ? z - 7 : z - 5;
	for (int i = 0; i < n; i ++) {
		int x = (z - 1 + d * hx[i]) / 2;
		int y = (z - 1 + d * hy[i]) / 2;
		set_state(x + y * z, Board::STATE_BLACK);
	}
}

string Board::loc_to_str(int loc) const
{
	ostringstream s;
	char c = 'a' + loc % z;
	if (c >= 'i') c ++;
	s << c << 1 + loc / z;
	return s.str();
}


// members of class Position
Position::Position()
{
	t = TURN_BLACK;
}

Position::Position(int size) :
	Board(size)
{
	t = TURN_BLACK;
}

Position::Position(const Position & position) : // copy constructor
	Board(position)
{
	t = position.t;
	k = position.k;
}

void Position::copy(const Position & position)
{
	Board::copy(position);
	t = position.t;
	k = position.k;
}

void Position::clear()
{
	Board::clear();
	t = TURN_BLACK;
	k.clear();
}

Position::Turn Position::get_turn() const
{
	return t;
}

void Position::set_turn(Turn turn)
{
	t = turn;
}

const set<int> & Position::get_ko() const
{
	return k;
}

void Position::set_ko(const set<int> & ko)
{
	k = ko;
}

bool Position::loc_is_ko(int loc) const
{
	return k.count(loc);
}

void Position::set_handi(int handi)
{
	Board::set_handi(handi);
	t = TURN_WHITE;
}

void Position::flip_turn()
{
	t = Turn(3 - t);
}

Position::Turn Position::flip_turn(Turn turn)
{
	return Turn(3 - turn);
}

// members of class Delta
Delta::Delta() :
	tf(false)
{
}

Delta::Delta(const Delta & delta)
{
	dl = delta.dl;
	tf = delta.tf;
	ok = delta.ok;
	nk = delta.nk;
}

void Delta::apply(Position & position) const
{
	for (vector<D>::const_iterator i = dl.begin(); i != dl.end(); i ++) {
		position.set_state(i->loc, i->ns);
	}
	if (tf) position.flip_turn();
	if (ok.size() || nk.size()) position.set_ko(nk);
}

void Delta::revert(Position & position) const
{
	if (ok.size() || nk.size()) position.set_ko(ok);
	if (tf) position.flip_turn();
	for (vector<D>::const_reverse_iterator i = dl.rbegin(); i != dl.rend(); i ++) {
		position.set_state(i->loc, i->os);
	}
}

void Delta::add_change(int loc, Board::State old_state, Board::State new_state)
{
	for (vector<D>::iterator i = dl.begin(); i != dl.end(); i ++) if (i->loc == loc) {
		msg(DBG_DEBUG) << "multiple add_change to the same site (" << i->loc << ")\n";
		assert(i->ns == old_state);
		if (i->os == new_state) { // back to old state, erase it!
			dl.erase(i);
		} else i->ns = new_state;
		return;
	}
	D d;
	d.loc = loc;
	d.os = old_state;
	d.ns = new_state;
	dl.push_back(d);
}

void Delta::del_change(int loc)
{
	for (vector<D>::iterator i = dl.begin(); i != dl.end(); i ++) if (i->loc == loc) {
		dl.erase(i);
		return;
	}
}

void Delta::set_turn_flip(bool turn_flip)
{
	tf = turn_flip;
}

void Delta::set_ko_old(set<int> old_ko)
{
	ok = old_ko;
}

void Delta::set_ko_new(set<int> new_ko)
{
	nk = new_ko;
}

bool Delta::get_turn_flip() const
{
	return tf;
}

bool Delta::is_null()
{
	return ! (dl.size() || tf || ok != nk);
}

bool Delta::is_changed(int loc)
{
	for (vector<D>::iterator i = dl.begin(); i != dl.end(); i ++) if (i->loc == loc) return true;
	return false;
}

Board::State Delta::get_new_state(int loc)
{
	for (vector<D>::iterator i = dl.begin(); i != dl.end(); i ++) if (i->loc == loc) return i->ns;
	throw Error("No change to loc");
}

int Delta::num_change()
{
	int d = 0;
	for (vector<D>::iterator i = dl.begin(); i != dl.end(); i ++) {
		d -= ((i->os + 1) % 3 - 1);
		d += ((i->ns + 1) % 3 - 1);
	}
	return d;
}

// members of class Prop
int Prop::the_id()
{
	return id();
}

int Prop::get_id()
{
	return id();
}

Prop * Prop::dup() const
{
	return new Prop;
}

Prop::Id::Id() :
	v(c ++)
{
}

int Prop::Id::operator()()
{
	return v;
}

int Prop::Id::c = 0;
Prop::Id Prop::id;

// members of class Comment
int Comment::the_id()
{
	return id();
}

int Comment::get_id()
{
	return id();
}

Prop * Comment::dup() const
{
	Comment * m = new Comment;
	m->c = c;
	return m;
}

const std::string & Comment::get_comment()
{
	return c;
}

void Comment::set_comment(const std::string & text)
{
	c = text;
}

Prop::Id Comment::id;

// members of class Marking
int Marking::the_id()
{
	return id();
}

int Marking::get_id()
{
	return id();
}

Prop * Marking::dup() const
{
	Marking * k = new Marking;
	k->m = m;
	return k;
}

void Marking::clear_marks()
{
	m.clear();
}

void Marking::add_mark(int loc, Mark mark)
{
	if (mark == MARK_NONE) {
		std::map<int, Mark>::iterator i = m.find(loc);
		if (i != m.end()) m.erase(i);
	} else m[loc] = mark;
}

Mark Marking::get_mark(int loc)
{
	std::map<int, Mark>::iterator i = m.find(loc);
	if (i != m.end()) return i->second;
	return MARK_NONE;
}

const std::map<int, Mark> & Marking::get_marks() const
{
	return m;
}

Prop::Id Marking::id;

// members of class Labels
int Labels::the_id()
{
	return id();
}

int Labels::get_id()
{
	return id();
}

Prop * Labels::dup() const
{
	Labels * k = new Labels;
	k->lb = lb;
	return k;
}

void Labels::clear_labels()
{
	lb.clear();
}

void Labels::add_label(int loc, const std::string & label)
{
	if (label == "") {
		std::map<int, std::string>::iterator i = lb.find(loc);
		if (i != lb.end()) lb.erase(i);
	} else lb[loc] = label;
}

const std::string & Labels::get_label(int loc)
{
	static const std::string n("");
	std::map<int, std::string>::const_iterator i = lb.find(loc);
	if (i != lb.end()) return i->second;
	return n;
}

const std::map<int, std::string> & Labels::get_labels() const
{
	return lb;
}

Prop::Id Labels::id;

// members of class Node
Node::Node(Node * parent) :
	m(MOVE_NONE),
	c(0)
{
	u = parent;
	if (u) {
		g = u->g; // copy game node pointer
		n = u->n;
	} else {
		g = this;
		n = 0;
	}
}

Node::~Node()
{
	while (d.size()) {
		delete d.back();
		d.pop_back();
	}
	if (c) delete c;
}

void Node::copy(const Node & node)
{
	while (d.size()) {
		delete d.back();
		d.pop_back();
	}
	for (vector<Node *>::const_iterator i = node.d.begin(); i != node.d.end(); i ++) {
		d.push_back(new Node(this));
		d.back()->copy(* (* i));
	}
	m = node.m;
	if (c) delete c;
	c = new Delta(* node.c);
	n = node.n;
	for (map<int, Prop *>::iterator i = p.begin(); i != p.end(); i ++) delete i->second;
	p.clear();
	for (map<int, Prop *>::const_iterator i = node.p.begin(); i != node.p.end(); i ++) p[i->first] = i->second->dup();
}

Node * Node::up()
{
	return u;
}

int Node::down_num()
{
	return d.size();
}

Node * Node::down(int index)
{
	return d[index];
}

Game * Node::game()
{
	return dynamic_cast<Game *>(g);
}

bool Node::set_move(int move, const Delta & delta)
{
	if (m != MOVE_NONE) n --;
	m = move;
	if (m != MOVE_NONE) n ++;
	if (c) delete c;
	c = new Delta(delta);
}

void Node::clear_move()
{
	m = MOVE_NONE;
	if (c) delete c;
	c = 0;
}

int Node::get_move() const
{
	return m;
}

const Delta * Node::get_delta() const
{
	return c;
}

Delta * Node::get_delta()
{
	return c;
}

int Node::add_child()
{
	d.push_back(new Node(this));
	return d.size() - 1;
}

int Node::get_index()
{
	if (! u) return - 1;
	for(int i = 0; i < u->d.size(); i ++) if (u->d[i] == this) return i;
}

void Node::del_child(Node * node)
{
	for (vector<Node *>::iterator i = d.begin(); i != d.end(); i ++) if (* i == node) {
		delete * i;
		d.erase(i);
		break;
	}
}

void Node::del_child(int index)
{
	vector<Node *>::iterator i = d.begin() + index;
	delete * i;
	d.erase(i);
}

int Node::get_num()
{
	return n;
}

void Node::set_num(int num)
{
	n = num;
}

Prop * Node::get_prop(int id)
{
	map<int, Prop *>::iterator i = p.find(id);
	if (i != p.end()) return i->second;
	return 0;
}

void Node::set_prop(int id, Prop * prop)
{
	map<int, Prop *>::iterator i = p.find(id);
	if (i != p.end()) {
		delete i->second;
		p.erase(i);
	}
	if (prop) p[prop->get_id()] = prop;
}

// members of class Walk
Walk::Walk() :
	n(0),
	cap_b(0),
	cap_w(0)
{
}

Walk::~Walk()
{
	if (n) n->game()->del_walk(this);
}

void Walk::set_node(Node * node)
{
	if (n) n->game()->del_walk(this);
	n = 0;
	if (! node) {
		clear();
		l = 0;
		n = 0;
		return;
	}
	n = node;
	init_game(n->game());
	// apply all deltas to recover the position
	Node * m = n;
	vector<Node *> s;
	l = - 1;
	while (m) {
		s.push_back(m);
		m = m->up();
		l ++;
	}
	while (s.size()) {
		apply_node(s.back());
		s.pop_back();
	}
}

Node * Walk::where() const
{
	return n;
}

void Walk::up()
{
	if (! n->up()) return;
	revert_node(n);
	n = n->up();
	l --;
}

void Walk::down(int index)
{
	if (index >= n->down_num()) return;
	n = n->down(index);
	l ++;
	apply_node(n);
}

int Walk::get_level()
{
	return l;
}

bool Walk::game_can_init()
{
	if (n != (Node *) n->game()) return false;
	if (n->down_num()) return false;
	return true;
}

int Walk::game_add_move(int move) // return child index
{
	Game * g = n->game();
	if (! g->check(* this, move)) return - 1;
	int i = n->add_child();
	n->down(i)->set_move(move, g->make_delta(* this, move));
	return i;
}

int Walk::game_add_setup()
{
	int i = n->add_child();
	return i;
}

void Walk::game_set_turn(Turn turn)
{
	assert(! n->down_num());
	assert(n->get_move() == MOVE_NONE);
	if (get_turn() == turn) return;
	Delta * d = n->get_delta();
	if (! d) {
		if (n == (Node *) n->game()) { // initial point
			n->game()->get_init()->set_turn(turn);
			flip_turn();
			return;
		}
		n->set_move(MOVE_NONE, Delta());
		d = n->get_delta();
		d->set_ko_old(get_ko());
		d->set_ko_new(get_ko());
	}
	d->set_turn_flip(! d->get_turn_flip());
	flip_turn();
	if (d->is_null()) n->clear_move();
}

void Walk::game_set_site(int loc, State state)
{
	assert(! n->down_num());
	assert(n->get_move() == MOVE_NONE);
	if (get_state(loc) == state) return;
	Delta * d = n->get_delta();
	if (! d) {
		if (n == (Node *) n->game()) { // starting point
			n->game()->get_init()->set_state(loc, state);
			set_state(loc, state);
			return;
		}
		n->set_move(MOVE_NONE, Delta());
		d = n->get_delta();
		d->set_ko_old(get_ko());
		d->set_ko_new(get_ko());
	}
	d->add_change(loc, get_state(loc), state);
	set_state(loc, state);
	if (d->is_null()) n->clear_move();
}

void Walk::game_set_handi(int handi)
{
	if (! game_can_init()) throw Error("can not change handicap");
	if (handi < 0) {
		n->game()->get_init()->clear(); // clear handicap
		clear();
	} else {
		n->game()->get_init()->set_handi(handi);
		set_handi(handi);
	}
	// set_node(n); // set to node again
}

void Walk::game_add_handi(int loc)
{
	if (! game_can_init()) throw Error("can not add handicap");
	n->game()->get_init()->set_state(loc, STATE_BLACK);
	set_state(loc, STATE_BLACK);
}

int Walk::get_b_cap()
{
	return cap_b;
}

int Walk::get_w_cap()
{
	return cap_w;
}

void Walk::init_game(Game * game)
{
	game->add_walk(this);
	if (! game->get_init()) {
		clear();
		return;
	};
	copy(* game->get_init());
	cap_b = 0;
	cap_w = 0;
}

void Walk::apply_node(Node * node)
{
	Delta * d = node->get_delta();
	if (d) {
		d->apply(* this);
		if (node->get_move() >= 0) {
			int c = d->num_change();
			switch (get_turn()) {
			case TURN_WHITE:
				if (c > 1) cap_b += (c - 1);
				else cap_w += (1 - c);
				break;
			case TURN_BLACK:
				if (c < - 1) cap_w -= (c + 1);
				else cap_b += (c + 1);
				break;
			}
		}
	}
}

void Walk::revert_node(Node * node)
{
	Delta * d = node->get_delta();
	if (d) {
		if (node->get_move() >= 0) {
			int c = d->num_change();
			switch (get_turn()) {
			case TURN_WHITE:
				if (c > 1) cap_b -= (c - 1);
				else cap_w -= (1 - c);
				break;
			case TURN_BLACK:
				if (c < - 1) cap_w += (c + 1);
				else cap_b -= (c + 1);
				break;
			}
		}
		d->revert(* this);
	}
}

// members of struct Rule
Rule::Rule()
{
	opt_overwrite_stone = false;
	opt_ignore_ko = false;
	opt_allow_suicide = false;
	opt_ko_style = KO_SIMPLE;
}

// members of class Game
Game::Game() :
	Node(0),
	p(0),
	h(- 1),
	km(0)
{
}

Game::~Game()
{
	vector<Walk *> ww = w;
	for (vector<Walk *>::iterator i = ww.begin(); i != ww.end(); i ++) {
		(* i)->set_node(0);
	}
	if (p) delete p;
}

void Game::copy(const Game & game)
{
	Node::copy(game);
}

void Game::set_init(const Position & position)
{
	p = new Position(position);
}

Position * Game::get_init()
{
	return p;
}

bool Game::check(const Walk & walk, int move)
{
	if (move == MOVE_PASS) return true; // pass is always allowed
	if (move == MOVE_NONE) return false; // must be a move
	if (! r.opt_overwrite_stone && walk.get_state(move) != Board::STATE_EMPTY) return false;
	if (! r.opt_ignore_ko) {
		if (walk.get_ko().count(move)) return false;
	}
	if (! r.opt_allow_suicide) {
		vector<int> k;
		walk.get_kill(move, k, Board::State(walk.get_turn()));
		if (k.size() && k[0] == move) return false;
	}
	return true;
}


Delta Game::make_delta(const Walk & walk, int move)
{
	Delta a;
	vector<int> k;
	if (move >= 0) { // put a stone
		a.add_change(move, walk.get_state(move), Board::State(walk.get_turn()));
		walk.get_kill(move, k, Board::State(walk.get_turn()));
		if (k.size()) {
			vector<int>::iterator i = k.begin();
			if (* i == move) { // suicide, change it back
				a.add_change(move, Board::State(walk.get_turn()), Board::STATE_EMPTY);
				i ++;
			}
			while (i != k.end()) {
				a.add_change(* i, walk.get_state(* i), Board::STATE_EMPTY);
				i ++;
			}
		}
	}
	// turn flips
	a.set_turn_flip(true);
	// ko change...
	a.set_ko_old(walk.get_ko());
	if (k.size() == 1) { // will kill one stone, check for simple ko
		vector<int> p = walk.neighbors(move);
		bool more = false;
		for (vector<int>::iterator i = p.begin(); i != p.end(); i ++) {
			Board::State s = walk.get_state(* i);
			if (s == Board::STATE_EMPTY || s == Board::State(walk.get_turn())) {
				more = true; // same stone around
				break;
			}
		}
		if (! more) { // all enemy around -> simple ko!
			set<int> ko;
			ko.insert(k[0]);
			a.set_ko_new(ko);
		}
	}
	return a;
}

const Rule & Game::get_rule() const
{
	return r;
}

void Game::set_rule(const Rule & rule)
{
	r = rule;
}

void Game::add_walk(Walk * walk)
{
	w.push_back(walk);
}

void Game::del_walk(Walk * walk)
{
	for (vector<Walk *>::iterator i = w.begin(); i != w.end(); i ++) if (walk == * i) {
		w.erase(i);
		return;
	}
	throw Error("can not find Walk in Game");
}

void Game::set_name(std::string name)
{
	nm = name;
}

const std::string & Game::get_name() const
{
	return nm;
}

void Game::set_komi(double komi)
{
	km = komi;
}

double Game::get_komi() const
{
	return km;
}
